"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AssetsPlugin = require("assets-webpack-plugin");
const CleanWebpackPlugin = require("clean-webpack-plugin");
const fs = require("fs");
const HtmlWebpackPlugin = require("html-webpack-plugin");
const path = require("path");
const UglifyJSPlugin = require("uglifyjs-webpack-plugin");
const webpack = require("webpack");
const webpack_bundle_analyzer_1 = require("webpack-bundle-analyzer");
const WorkboxPlugin = require("workbox-webpack-plugin");
const target_config_1 = require("./target.config");
exports.HMR = () => {
    if (target_config_1.isDevelopment) {
        return [
            new webpack.NamedModulesPlugin(),
            new webpack.HotModuleReplacementPlugin(),
        ];
    }
    return [];
};
exports.extractVendors = () => ([]);
exports.uglify = () => {
    if (!target_config_1.isDevelopment) {
        return [
            new UglifyJSPlugin({
                sourceMap: target_config_1.isDebug,
                uglifyOptions: {
                    cache: true,
                    compress: {
                        sequences: true,
                        unused: true,
                    },
                    ecma: 8,
                    parallel: 4,
                },
            }),
        ];
    }
    return [];
};
exports.analyze = () => {
    if (target_config_1.isDebug) {
        return [
            new webpack_bundle_analyzer_1.BundleAnalyzerPlugin({
                analyzerPort: target_config_1.analyze.port,
            }),
        ];
    }
    return [];
};
exports.htmlWebpackPlugin = (dll = true) => {
    let dllConfig = { [target_config_1.dll_entry_name]: {} };
    if (dll && fs.existsSync(target_config_1.dll_assets_path)) {
        dllConfig = require(target_config_1.dll_assets_path);
    }
    return [
        new HtmlWebpackPlugin({
            dllConfig,
            filename: "index.html",
            inject: false,
            template: path.resolve(__dirname, "../../../../templates/html-webpack-template.ejs"),
        }),
    ];
};
exports.DllPlugin = () => {
    return [
        new webpack.DllPlugin({
            context: target_config_1.context,
            name: target_config_1.dll_entry_name,
            path: target_config_1.dll_manifest_path,
        }),
        new webpack.optimize.OccurrenceOrderPlugin(false),
        new AssetsPlugin({
            filename: target_config_1.dll_assets_name,
            fullPath: false,
            path: target_config_1.dll_path,
            prettyPrint: true,
        }),
    ];
};
exports.DllReferencePlugin = () => {
    if (fs.existsSync(target_config_1.dll_manifest_path)) {
        return [
            new webpack.DllReferencePlugin({
                context: target_config_1.context,
                manifest: target_config_1.dll_manifest_path,
                name: target_config_1.dll_lib_name,
            }),
        ];
    }
    return [];
};
exports.workboxPlugin = () => {
    if ((!target_config_1.isDevelopment || target_config_1.isBuild) && !target_config_1.isStart) {
        console.log(target_config_1.output.path);
        return [
            new WorkboxPlugin.GenerateSW({
                clientsClaim: true,
                globDirectory: target_config_1.output.path,
                globPatterns: ["*.{js,html}"],
                importWorkboxFrom: "local",
                maximumFileSizeToCacheInBytes: 10 * 1024 * 1024,
                navigateFallback: "/",
                skipWaiting: true,
                swDest: "sw.js",
            }),
        ];
    }
    return [];
};
exports.clean = (toRemove = [], force = false) => {
    if (!target_config_1.isDevelopment || force || target_config_1.isBuild) {
        return [new CleanWebpackPlugin(toRemove, { root: target_config_1.root, verbose: target_config_1.isDebug })];
    }
    return [];
};
