"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chalk_1 = require("chalk");
const chokidar = require("chokidar");
const copy = require("copy");
const del = require("del");
const fs = require("fs");
const mkdirp = require("mkdirp");
const path = require("path");
const target_config_1 = require("./target.config");
const watchers = [];
function npmHardSyncStart(watch = true) {
    return tslib_1.__awaiter(this, void 0, void 0, function* () {
        console.log("");
        console.log(chalk_1.default.bold.yellow("[ NPM_HARD_SYNC ]"));
        const npmHardSyncTarget = target_config_1.npmHardSyncLinks.map(target_config_1.resolveGlobalNodeModules);
        for (const npmLink of npmHardSyncTarget) {
            if (!fs.existsSync(npmLink)) {
                console.log(chalk_1.default.bold.red(`You havn't link in global space on ${npmLink}`));
                console.log(chalk_1.default.bold.cyan(`You need go to the project and do npm link;`));
                throw new Error(`Symlink not found: ${npmLink}`);
            }
            else {
                console.log(chalk_1.default.bold.yellow(`[ ORIGIN_SYMLINK: ${npmLink} ]`));
            }
        }
        const npmHardSyncCopyTarget = npmHardSyncTarget.map((modulePath, index) => {
            const copiesTarget = {
                moduleName: target_config_1.npmHardSyncLinks[index],
                paths: [],
            };
            const pkgPath = path.resolve(modulePath, "package.json");
            if (fs.existsSync(pkgPath)) {
                copiesTarget.paths.push(pkgPath);
                const { files } = require(path.resolve(modulePath, "package.json"));
                for (const file of files) {
                    const filePath = path.resolve(modulePath, file);
                    if (fs.existsSync(filePath)) {
                        copiesTarget.paths.push(`${filePath}/**/*.d.ts`);
                        copiesTarget.paths.push(`${filePath}/**/*.js`);
                    }
                }
            }
            return copiesTarget;
        });
        const LOCAL_NODE_MODULES = path.resolve(target_config_1.root, "node_modules");
        const localCopies = target_config_1.npmHardSyncLinks.map((link) => path.resolve(LOCAL_NODE_MODULES, link));
        for (const localCopy of localCopies) {
            yield new Promise((resolve, reject) => {
                if (fs.existsSync(localCopy)) {
                    del(localCopy).then((paths) => {
                        for (const item of paths) {
                            console.log(chalk_1.default.yellow("[ REMOVED_LOCAL_COPY ]"), chalk_1.default.cyan(item));
                        }
                        mkdirp(localCopy, (error) => {
                            if (error) {
                                console.error(error);
                                reject();
                            }
                            else {
                                console.log(chalk_1.default.yellow("[ CREATED_DIR ]", chalk_1.default.cyan(`${localCopy}`)));
                                resolve();
                            }
                        });
                    });
                }
                else {
                    mkdirp(localCopy, (error) => {
                        if (error) {
                            console.error(error);
                            reject();
                        }
                        else {
                            console.log(chalk_1.default.yellow("[ CREATED_DIR ]", chalk_1.default.cyan(`${localCopy}`)));
                            resolve();
                        }
                    });
                }
            });
        }
        for (const { moduleName, paths } of npmHardSyncCopyTarget) {
            yield new Promise((resolve, reject) => {
                copy(paths, path.resolve(LOCAL_NODE_MODULES, moduleName), (error, files) => {
                    if (!error) {
                        if (target_config_1.isDebug) {
                            for (const file of files) {
                                console.log(chalk_1.default.yellow(`[ COPIED ][ ${moduleName} ]`), chalk_1.default.cyan(file.path));
                            }
                        }
                        else {
                            console.log(chalk_1.default.yellow(`[ COPIED ][ ${moduleName} ]`), chalk_1.default.cyan(`[ ${files.length} FILES ]`));
                        }
                        resolve();
                    }
                    else {
                        console.error(error);
                        reject();
                    }
                });
            });
        }
        if (watch) {
            console.log(chalk_1.default.yellow("[ WATCH_INIT ]"));
            if (watchers.length > 0) {
                for (const watcher of watchers) {
                    watcher.close();
                }
            }
            for (const { moduleName, paths } of npmHardSyncCopyTarget) {
                const watcher = chokidar.watch(paths);
                watcher.on("change", (from) => {
                    const insidePath = from.substr(from.indexOf(moduleName) + moduleName.length + 1, from.length);
                    const to = path.resolve(LOCAL_NODE_MODULES, moduleName, insidePath);
                    fs.unlinkSync(to);
                    if (target_config_1.isDebug) {
                        console.log("");
                        console.log(chalk_1.default.yellow("[ REMOVED_LOCAL_COPY ]"));
                        console.log(chalk_1.default.cyan(`[ OF: ${to} ]`));
                    }
                    fs.copyFileSync(from, to);
                    if (target_config_1.isDebug) {
                        console.log("");
                        console.log(chalk_1.default.yellow(`[ COPIED ][ ${moduleName} ]`));
                        console.log(chalk_1.default.cyan(`[ FROM: ${from} ]`));
                        console.log(chalk_1.default.cyan(`[ TO ${to} ]`));
                    }
                    if (!target_config_1.isDebug) {
                        console.log(chalk_1.default.yellow(`[ NPM_HARD_SYNC ][ UPDATE ][ ${insidePath} ]`));
                    }
                });
                watchers.push(watcher);
            }
        }
    });
}
exports.npmHardSyncStart = npmHardSyncStart;
