"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
const index_1 = require("../source/index");
const CWD = process.cwd();
const config = index_1.default();
exports.resolveGlobalNodeModules = (name = "") => path.resolve(process.env.REARGUARD_GLOBAL_NODE_MODULES_PATH || "", name);
exports.resolveNodeModules = (name = "") => path.resolve(process.env.REARGUARD_NODE_MODULE_PATH || "", name);
exports.resolveTarget = (relPath = "") => path.resolve(CWD, relPath);
exports.isDevelopment = config.isDevelopment;
exports.isDebug = config.isDebug;
exports.isStart = config.isStart;
exports.isBuild = config.isBuild;
exports.nodeModulePath = config.nodeModulePath;
exports.socket = config.socket;
exports.root = exports.resolveTarget();
exports.context = exports.resolveTarget(config.context);
exports.entry = config.entry;
exports.output = {
    chunkFilename: exports.isDevelopment
        ? "[name].chunk.js?[hash:8]"
        : "[chunkhash:32].chunk.js",
    filename: exports.isDevelopment ? "[name].js?[hash:8]" : "[chunkhash:32].js",
    globalObject: "this",
    path: exports.resolveTarget(config.output.path),
    pathinfo: exports.isDebug,
    publicPath: config.output.publicPath,
};
exports.modules = [
    ...config.modules.map((relPath) => exports.resolveTarget(relPath)),
    exports.resolveTarget("node_modules"),
    exports.resolveNodeModules(),
];
exports.stats = exports.isDebug
    ? "verbose"
    : {
        assets: true,
        colors: true,
        context: exports.context,
        hash: true,
        modules: false,
        performance: false,
        publicPath: true,
        timings: true,
        version: true,
    };
exports.proxy = config.proxy;
exports.WDSConfig = {
    bonjour: true,
    compress: true,
    contentBase: exports.resolveTarget(path.resolve(exports.root, "dll", exports.isDevelopment ? "dev" : "prod")),
    historyApiFallback: true,
    hot: true,
    https: true,
    open: true,
    overlay: false,
    proxy: exports.proxy,
    publicPath: exports.output.publicPath,
    stats: exports.stats,
};
exports.env = {
    NODE_ENV: JSON.stringify(exports.isDevelopment ? "development" : "production"),
};
exports.analyze = {
    port: config.analyze.port,
};
exports.ts = config.typescript.config;
exports.tsConfigPath = exports.resolveTarget(config.typescript.configPath);
exports.tsLintConfigPath = exports.resolveTarget("tslint.json");
const externalPluginsPath = exports.resolveTarget(config.postCSS.plugins);
exports.postCSS = {
    config: require(path.resolve(__dirname, "postcss.config.js")),
    plugins: {
        list: fs.existsSync(externalPluginsPath)
            ? require(externalPluginsPath)
            : [],
        path: externalPluginsPath,
    },
};
exports.pkg = {
    dependencies: config.dependencies,
    engines: config.engines,
    nodeVersion: config.nodeVersion,
};
exports.dll_path = path.resolve(path.join(exports.root, "dll", exports.isDevelopment ? "dev" : "prod"));
exports.dll_manifest_name = "vendor-manifest.json";
exports.dll_assets_name = "vendor-hash.json";
exports.dll_manifest_path = path.resolve(exports.dll_path, exports.dll_manifest_name);
exports.dll_assets_path = path.join(exports.dll_path, exports.dll_assets_name);
exports.dll_lib_name = "dll_vendor";
exports.dll_lib_file_name = "dll.vendor.[hash].js";
exports.dll_lib_output_path = path.join(exports.root, "dll", exports.isDevelopment ? "dev" : "prod");
exports.dll_entry_name = "vendors";
exports.npmHardSyncLinks = config.npmHardSync;
