"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const fs = require("fs");
const path = require("path");
const target_config_1 = require("./target.config");
exports.default = () => {
    console.log("");
    const { compilerOptions, compileOnSave } = target_config_1.ts;
    const { devDependencies: { typescript: version }, } = require(path.resolve(__dirname, "../../../../package.json"));
    const pathKeys = Object.keys(compilerOptions.paths);
    const paths = {};
    for (const pathKey of pathKeys) {
        paths[pathKey] = compilerOptions.paths[pathKey].map((P) => target_config_1.resolveTarget(P));
    }
    const config = {
        compileOnSave,
        compilerOptions: Object.assign({
            allowJs: false,
            checkJs: false,
            target: "es6",
            module: "es6",
            pretty: true,
            lib: [
                "es6",
                "es7",
                "dom",
            ],
            jsx: "react",
            declaration: false,
            importHelpers: true,
            downlevelIteration: false,
            isolatedModules: false,
            allowSyntheticDefaultImports: true,
            removeComments: true,
            preserveConstEnums: true,
            noEmit: false,
            rootDir: "./",
            charset: "utf8",
            noErrorTruncation: true,
            diagnostics: target_config_1.isDebug,
            sourceMap: target_config_1.isDebug,
            strict: true,
            alwaysStrict: true,
            strictNullChecks: true,
            strictFunctionTypes: true,
            strictPropertyInitialization: true,
            noFallthroughCasesInSwitch: true,
            noImplicitAny: true,
            noImplicitReturns: true,
            noImplicitThis: true,
            noUnusedLocals: true,
            noUnusedParameters: true,
            moduleResolution: "node",
            baseUrl: target_config_1.context,
            rootDirs: [],
            paths: {},
            typeRoots: [
                "node_modules/@types",
            ],
            types: [],
            forceConsistentCasingInFileNames: true,
            experimentalDecorators: true,
        }, compilerOptions, { paths }),
        exclude: ["node_modules", "node_modules/.cache"],
        include: [target_config_1.context],
        version,
    };
    const tsLint = {
        defaultSeverity: "error",
        extends: ["tslint:recommended", "tslint-no-circular-imports"],
        jsRules: {},
        rules: {
            "max-classes-per-file": [false],
            "member-access": true,
            "no-console": false,
            "no-var-requires": false,
            "variable-name": false,
        },
        rulesDirectory: [],
    };
    fs.writeFileSync(target_config_1.tsConfigPath, JSON.stringify(config, null, 2));
    fs.writeFileSync(target_config_1.tsLintConfigPath, JSON.stringify(tsLint, null, 2));
    console.log(chalk_1.default.bold.cyanBright(`[Build typescript and tslint config]`.toUpperCase()));
    console.log(chalk_1.default.cyan(`TSConfig: ${target_config_1.tsConfigPath}`));
    console.log(chalk_1.default.cyan(`TSLint: ${target_config_1.tsLintConfigPath}`));
    console.log(``);
};
